#include "stdafx.h"
#include "Utility.hpp"

namespace Lmt {
	bool Utility::CreateDirectory( const wstring &path ) {
		wstring targetPath = Utility::AddBackSlash( path );
		string realPath = Utility::WStringToString( targetPath );
		return ::MakeSureDirectoryPathExists( realPath.c_str() ) ? true : false;
	}

	bool Utility::IsDirectoryExists( const wstring &path ) {
		DWORD attrs = ::GetFileAttributes( path.c_str() );
		return attrs != -1 && (attrs & FILE_ATTRIBUTE_DIRECTORY) != 0;
	}

	bool Utility::IsExtensionMatched( const wstring &path, const wstring &ext ) {
		wstring pathExt = Utility::GetExtensionFromPath( path );
		for (wstring::iterator it = pathExt.begin(); it != pathExt.end(); it++)
			*it = towlower( *it );
		return pathExt == ext;
	}

	void Utility::AntiOverRun( char *str, size_t length ) {
		if (length > 0) {
			for (size_t i = 0; i < length; i++) {
				if (*str == '\0') {
					return;
				}
			}
			str[length - 1] = '\0';
		}
	}

	wstring Utility::GetProgramFilesDirectory() {
		wchar_t buffer[MAX_PATH];
		if (!::SHGetSpecialFolderPath( NULL, buffer, CSIDL_PROGRAM_FILES, FALSE )) {
			wcscpy( buffer, L"C:\\Program Files\\" );
		}
		return buffer;
	}

	wstring Utility::GetCurrentDirectory() {
		wchar_t buffer[MAX_PATH];
		buffer[0] = '\0';
		::GetCurrentDirectory( sizeof(buffer), buffer );
		return buffer;
	}

	wstring Utility::CombinePath( const wstring &left, const wstring &right ) {
		wchar_t *buffer = new wchar_t [left.length() + right.length() + 4];
		wstring result;
		::PathCombine( buffer, left.c_str(), right.c_str() );
		result = buffer;
		delete [] buffer;
		return result;
	}

	wstring Utility::AddBackSlash( const wstring &path ) {
		size_t length = path.length() + 4;
		wchar_t *buffer = new wchar_t [length];
		wstring result;
		wcscpy( buffer, path.c_str() );
		::PathAddBackslash( buffer );
		result = buffer;
		delete [] buffer;
		return result;
	}

	wstring Utility::AddExtension( const wstring &path, const wstring &extension ) {
//		size_t length = path.length() + extension.length() + 4;
//		wchar_t *buffer = new wchar_t [length];
//		wstring result;
//		wcscpy_s( buffer, length, path.c_str() );
//		::PathAddExtension( buffer, extension.c_str() );
//		result = buffer;
//		delete [] buffer;
//		return result;
		return path + extension;
	}

	wstring Utility::ChangeExtension( const wstring &source, const wstring &newExt ) {
		wstring result;
		wstring ext = (newExt.length() > 0 && newExt[0] != L'.') ? L"." : L"";
		size_t length = source.length() + newExt.length() + 10;
		wchar_t *buffer = new wchar_t[length];

		wcscpy( buffer, source.c_str() );
		::PathRemoveExtension( buffer );
		ext += newExt;
		if (ext.length() > 0) {
			wcscat( buffer, ext.c_str() );
		}

		result = buffer;
		delete [] buffer;
		return result;
	}

	wstring Utility::GetFileNameFromPath( const wstring &path ) {
		return ::PathFindFileName( path.c_str() );
	}

	wstring Utility::GetDirectoryFromPath( const wstring &path ) {
		return wstring( path.c_str(), path.length() - Utility::GetFileNameFromPath( path ).length() );
	}

	wstring Utility::GetExtensionFromPath( const wstring &path ) {
		return ::PathFindExtension( path.c_str() );
	}

	wstring Utility::GetModuleDirectory( void ) {
		wchar_t path[MAX_PATH];
		::GetModuleFileName( NULL, path, sizeof(path) );
		return Utility::GetDirectoryFromPath( path );
	}

	string Utility::WStringToString( const wstring &s ) {
		size_t length = s.length() * 2 + 1;
		char *buffer = new char [length];
		size_t realLength = wcstombs( buffer, s.c_str(), length );
		string result( buffer, realLength == -1 ? 0 : realLength );
		delete [] buffer;
		return result;
	}

	wstring Utility::StringToWString( const string &s ) {
		size_t length = s.length() + 1;
		wchar_t *buffer = new wchar_t [length];
		size_t realLength = mbstowcs( buffer, s.c_str(), s.length() );
		wstring result( buffer, realLength == -1 ? 0 : realLength );
		delete [] buffer;
		return result;
	}

	wstring Utility::EncodeToUrl( const wstring &s ) {
		wstring result;
		for (wstring::const_iterator it = s.begin(); it != s.end(); it++) {
			if (*it == L'%') {
				result += L"%25";
			}
			else {
				result += *it;
			}
		}
		return result;
	}

	HWND Utility::GetConsoleWindow() {
		HWND hwnd;
		wchar_t newTitle[1024];
		wchar_t oldTitle[1024];

		if (::GetConsoleTitle( oldTitle, sizeof(oldTitle) ) == 0) {
			return NULL;
		}
		else {
			swprintf( newTitle, L"%d/%d", ::GetTickCount(), ::GetCurrentProcessId() );
			::SetConsoleTitle( newTitle );
			::Sleep( 40 );
			hwnd = ::FindWindow( NULL, newTitle );
			::SetConsoleTitle( oldTitle );
			return hwnd;
		}
	}

	unsigned int Utility::GetFileSize( const wstring &path ) {
		DWORD result;
		DWORD attr;
		WIN32_FIND_DATA fd;
		HANDLE hfind;

		attr = ::GetFileAttributes( path.c_str() );
		if (attr == ~0 || (attr == FILE_ATTRIBUTE_DIRECTORY) != 0) {
			return 0xffffffff;
		}

		hfind = ::FindFirstFile( path.c_str(), &fd );
		if (hfind == INVALID_HANDLE_VALUE) {
			return 0xffffffff;
		}

		if (fd.nFileSizeHigh > 0) {
			result = 0xffffffff;
		}
		else {
			result = fd.nFileSizeLow;
		}

		::FindClose( hfind );
		return result;
	}

	int Utility::GreatestCommonDivisor( int m, int n ) {
		int temp;

		//[Nbȟݏ@
		//u]ŊvƂłȂȂ܂ŌJԂ葱
		while ((m % n) != 0) {
			temp = n;
			n = m % n;
			m = temp;
		}

		return n;
	}

	int Utility::LeastCommonMultiple( int m, int n ) {
		//(ŏ{) = m * n / (ő)
		return (m * n) / GreatestCommonDivisor( m, n ); 
	}
}
